`timescale 1ns/1ps
/*******************************************************************************
 * File Name:		VerilogCSP.v
 * Written by:		Arash Saifhashemi, saifhash[at]usc[dot]edu
 * Last update:		March 27, 2006 - Rev. 2.10, 
 * SEND_REQ and RECEIVE_REQ added. Half buffer macros added
 *
 * 		USC Asynchronous CAD/VLSI gourp: http://jungfrau.usc.edu/
 ******************************************************************************/

`ifdef __CHAN_DEF__
`else
 //Change the values of the following macros to set the delay of Send and Receive
   `define DEF_SEND_DEL      0
   `define DEF_RECEIVE_DEL   0

	`define __CHAN_DEF__
	
	`define SEND(_prt_,_value_)    begin\
	   #`DEF_SEND_DEL;\
		force _prt_={_value_,_prt_[1],1'b1};\
		wait (_prt_[1]==1'b1);\
		force _prt_={_value_,_prt_[1],1'b0};\
		wait (_prt_[1]==1'b0);\
		#0;\
	end
 
	`define RECEIVE(_prt_,_value_)    begin\
      #`DEF_RECEIVE_DEL;\
		wait (_prt_[0]==1'b1);\
		{_value_,dummy}=_prt_;\
		force _prt_[1]=1'b1;\
		wait (_prt_[0]==1'b0);\
		force _prt_[1]=1'b0;\
		#0;\
	end

	`define RECEIVE_SEND_HALF1(_prt1_,_value_,_prt2_)    begin\
      force _prt2_[0]=1'b0;\
		force _prt1_[1]=1'b0;\
		wait (_prt1_[0]==1'b1 && _prt2_[1]!==1'b1);\
		{_value_,dummy}=_prt1_;\
		end
	
   	`define RECEIVE_SEND_HALF2(_prt1_,_value_,_prt2_)    begin\
		force _prt2_={_value_,_prt2_[1],1'b1};\
		force _prt1_[1]=1'b1;\
		wait (_prt1_[0]==1'b0 && _prt2_[1]==1'b1);\
		end



`define SEND_REQ(_prt_)    begin\
	   #`DEF_SEND_DEL;\
		force _prt_[0]=1'b1;\
		wait (_prt_[1]==1'b1);\
		force _prt_[0]=1'b0};\
		wait (_prt_[1]==1'b0);\
		#0;\
	end
 
	`define RECEIVE_REQ(_prt_,_value_)    begin\
      #`DEF_RECEIVE_DEL;\
		wait (_prt_[0]==1'b1);\
		force _prt_[1]=1'b1;\
		wait (_prt_[0]==1'b0);\
		force _prt_[1]=1'b0;\
		#0;\
	end	

	`define PROBE(_prt_)   (_prt_[0]===1'b1)
 
	`define OUTPORT(_prt_,_lenght_) output[_lenght_+1:0] _prt_;
	
	`define INPORT(_prt_,_lenght_) input[_lenght_+1:0] _prt_;
  
	`define CHANNEL(_ch_,_lenght_) wire [_lenght_+1:0] _ch_;
	
	`define USES_CHANNEL reg [1:0] dummy; 
	
	`define INPUT(_mod_,_modPrt_,_actPrt_, _width_, _slack_, _BL_, _FL_)	input[_width_+1:0] _modPrt_;\
		wire [_width_+1:0] _actPrt_;\
		port	 #(_width_, _slack_, _BL_, _FL_) _mod_ (_modPrt_, _actPrt_);
		
	`define OUTPUT(_mod_,_modPrt_,_actPrt_, _width_, _slack_, _BL_, _FL_)	output[_width_+1:0] _modPrt_;\
		wire [_width_+1:0] _actPrt_;\
		port	 #(_width_,_slack_, _BL_, _FL_) _mod_ (_actPrt_, _modPrt_);
	
`endif